#include <IRSendRev.h>
#include <Wire.h>

#include "Rainbow.h"

#define IRPIN  2

unsigned char DispMode = DISPTIME;

const unsigned char number[10] =
{
    0x7b,0x12,0x67,0x37,0x1e,0x3d,0x7d,0x13,0xff,0xbf
};//0~9

const unsigned char drive[9] =
{
    0,1,2,4,5,6
};//line
unsigned char PowerKey = 0;

unsigned char driveLine=0;
unsigned char driveSeg=0;

unsigned char LTopNumber    = 0;
unsigned char LBottomNumber = 0;
unsigned char RTopNumber    = 0;
unsigned char RBottomNumber = 0;

unsigned char LTopNumberScore    = 0;
unsigned char LBottomNumberScore = 0;
unsigned char RTopNumberScore    = 0;
unsigned char RBottomNumberScore = 0;

unsigned char LTopNumberTime     = 0;
unsigned char LBottomNumberTime  = 0;
unsigned char RTopNumberTime     = 0;
unsigned char RBottomNumberTime  = 0;

unsigned char Lred      = number[0];//Left Top Red segment
unsigned char LRred     = number[0];//Left Bottom Right Red segment
unsigned char LLred     = number[0];//Left Bottom Left Red segment
unsigned char Rgreen    = number[0];//Right Top Red segment
unsigned char RRgreen   = number[0];//Right Bottom Right Red segment
unsigned char RLgreen   = number[0];//Right Bottom Left Red segment

//**************************time******************************
unsigned char Hour      = 0;
unsigned char Minute    = 0;
unsigned char HourBuf   = 0;
unsigned char MinuteBuf = 0;

//**************************auto turn off*********************
long AutoTime       = 0;
long AutoTimeBuf    = 0;

//=============================================================
void init_timer1(void)          //initialize Timer1 to 1ms overflow
{
    TCCR1A = 0;                 // clear control register A
    TCCR1B = _BV(WGM13);        // set mode as phase and frequency correct pwm, stop the timer
    ICR1=10000;                 //(XTAL * microseconds) / 2000000  1ms cycles
    TIMSK1 = _BV(TOIE1);
    TCNT1 = 0;
    TCCR1B |= _BV(CS10);
    sei();                      //enable global interrupt
}
//=============================================================
ISR(TIMER1_OVF_vect)          //Timer1 Service
{
    close_all_line;
    if(PowerKey)
    {
        if(driveLine>5)
        {
            driveLine=0;
        }
        shiftData(drive[driveLine]);
        open_line(drive[driveLine]);
        driveLine++;
    }
}

void setup()
{
    DDRD=0xff;
    DDRC=0xff;
    DDRB=0xff;
    PORTD=0;
    PORTB=0;
#if __Debug
    Serial.begin(38400);
#endif
    Wire.begin();
    IR.Init(IRPIN);
    close_all_line;
    shift_24_bit(0,0,0);
    init_timer1();
}

void loop()
{
    unsigned char keyLbyte=0;

    unsigned char dataIr[20] = {};
    for(;;)
    {
        if(IR.IsDta())
        {
            AutoTimeBuf = AutoTime;
            IR.Recv(dataIr);
            figureKey(dataIr[dataIr[0]]);
        }
        getTime(&Hour, &Minute);
        if((Minute != MinuteBuf) && (DispMode == DISPTIME)) // refresh time
        {
            Serial.println("time");
            MinuteBuf       = Minute;
            HourBuf         = Hour;
            LBottomNumberTime   = HourBuf;
            RBottomNumberTime   = MinuteBuf;

            LBottomNumber       = LBottomNumberTime;
            RBottomNumber       = RBottomNumberTime;
            DispChange();
        }
     /*   
        if(PowerKey == 1)
        {
            AutoTime = millis();
            if((AutoTime - AutoTimeBuf) > AUTOTURNOFFTIME*1000)  // auto turn off
            {
                figureKey(POWER);
            }
        }*/
    }
}
//==============================================================
void shift_8_bit(unsigned char LS)
{
    for(unsigned char i=0;i<8;i++)
    {
        if(LS&0x80)
        {
            shift_data_1;
        }
        else
        {
            shift_data_0;
        }
        clk_rising;
        LS<<=1;
    }
}

void shiftData(unsigned char data)
{
    switch(data)
    {
        case 0:
        {
            shift_24_bit(0,0,Lred);
            break;
        }
        case 1:
        {
            shift_24_bit(0,LRred,0);
            break;
        }
        case 2:
        {
            shift_24_bit(LLred,0,0);
            break;
        }
        case 4:
        {
            shift_24_bit(0,0,Rgreen);
            break;
        }
        case 5:
        {
            shift_24_bit(0,RRgreen,0);
            break;
        }
        case 6:
        {
            shift_24_bit(RLgreen,0,0);
            break;
        }
    }
}
//==============================================================
void shift_24_bit(unsigned char ch1,unsigned char ch2,unsigned char ch3)
{
    disable_oe;
    le_high;
    shift_8_bit(ch1);
    shift_8_bit(ch2);
    shift_8_bit(ch3);
    le_low;
    enable_oe;
}
//==============================================================
void open_line(unsigned char line)
{
    switch(line&0x07)
    {
        case 0:
        {
            open_line0;
            break;
        }
        case 1:
        {
            open_line1;
            break;
        }
        case 2:
        {
            open_line2;
            break;
        }
        case 3:
        {
            open_line3;
            break;
        }
        case 4:
        {
            open_line4;
            break;
        }
        case 5:
        {
            open_line5;
            break;
        }
        case 6:
        {
            open_line6;
            break;
        }
        case 7:
        {
            open_line7;
            break;
        }
    }
}

void figureKey(unsigned char Key)
{
#if __Debug
    Serial.print("Key = ");Serial.println(Key);
#endif
    if(Key == POWER)
    {
        PowerKey = 1 - PowerKey;
        close_all_line;
        shift_24_bit(0,0,0);
    }
    if(PowerKey == 0)
    {
        return;
    }

    if(Key == MODE)
    {
        DispMode = 1 - DispMode;

    }
    else if(Key == TEST)
    {
        getTime(&Hour, &Minute);
#if __Debug
        Serial.print(Hour);
        Serial.print(" : ");
        Serial.println(Minute);
#endif
        return;
    }

    if(DispMode == DISPSCORE)
    switch(Key)
    {
        case LUP://Lup;
        {
            LBottomNumberScore++;
            if(LBottomNumberScore>99)
            {
                LBottomNumberScore=99;
            }
            break;
        }
        case LDOWN://Ldown:
        {
            LBottomNumberScore--;
            if(LBottomNumberScore == 0xff)
            {
                LBottomNumberScore=0;
            }
            break;
        }
        case RUP://Rup:
        {
            RBottomNumberScore++;
            if(RBottomNumberScore>99)
            {
                RBottomNumberScore=99;
            }
            break;
        }
        case RDOWN://Rdown:
        {
            RBottomNumberScore--;
            if(RBottomNumberScore==0xff)
            {
                RBottomNumberScore=0;
            }
            break;
        }
        case GAME://game:
        {
            if(LBottomNumberScore>RBottomNumberScore)
            {
                LTopNumberScore++;
                LBottomNumberScore = 0;
                RBottomNumberScore = 0;
            }
            else if(LBottomNumberScore<RBottomNumberScore)
            {
                RTopNumberScore++;
                LBottomNumberScore = 0;
                RBottomNumberScore = 0;
            }
            break;
        }
        case CLEAR://clear:
        {
            LTopNumberScore     = 0;
            LBottomNumberScore  = 0;
            RTopNumberScore     = 0;
            RBottomNumberScore  = 0;
            break;
        }

        default:
#if __Debug
        Serial.println("error key!");
#else
        ;
#endif
    }

    if(DispMode == DISPSCORE)
    {
#if __Debug
        Serial.println("change mode to score");
#endif
        LTopNumber     = LTopNumberScore;
        LBottomNumber  = LBottomNumberScore;
        RTopNumber     = RTopNumberScore;
        RBottomNumber  = RBottomNumberScore;
    }
    else
    {
#if __Debug
        Serial.println("change mode to time");
#endif
        LTopNumber     = LTopNumberTime;
        LBottomNumber  = LBottomNumberTime;
        RTopNumber     = RTopNumberTime;
        RBottomNumber  = RBottomNumberTime;
    }
    DispChange();
}

void DispChange()
{
    Lred = LTopNumber%10;
    Lred = number[Lred];
    LRred = LBottomNumber%10;
    LRred = number[LRred];
    LLred = (LBottomNumber%100)/10;
    LLred = number[LLred];

    Rgreen = RTopNumber%10;
    Rgreen = number[Rgreen];
    RRgreen = RBottomNumber%10;
    RRgreen = number[RRgreen];
    RLgreen = (RBottomNumber%100)/10;
    RLgreen = number[RLgreen];
}
// Convert binary coded decimal to normal decimal numbers
byte bcdToDec(byte val)
{
    return ( (val/16*10) + (val%16) );
}

void getTime(unsigned char * hour, unsigned char * minute)
{
    // Reset the register pointer
    Wire.beginTransmission(DS1307_I2C_ADDRESS);
    Wire.write(0x00);
    Wire.endTransmission();

    Wire.requestFrom(DS1307_I2C_ADDRESS, 7);

    // A few of these need masks because certain bits are control bits

    byte second     = bcdToDec(Wire.read() & 0x7f);
    *minute         = bcdToDec(Wire.read());
    *hour           = bcdToDec(Wire.read() & 0x3f);  // Need to change this if 12 hour am/pm
    byte dayOfWeek  = bcdToDec(Wire.read());
    byte day        = bcdToDec(Wire.read());
    byte month      = bcdToDec(Wire.read());
    byte year       = bcdToDec(Wire.read());

}

/*********************************************************************************************************
  END FILE
*********************************************************************************************************/

